package writer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import grammar.Grammar;
import grammar.Type2Grammar;
import type2.Type2Parser;
import type3.Type3Parser;
import word_generator.WordGenerator;

public class Writer {

	private static final int DISTINCT_WORDS = 5;
	private static final int MAX_LENGTH = 5;

	WordGenerator wg = new WordGenerator();
	Type2Parser t2p = new Type2Parser();
	Type3Parser t3p = new Type3Parser();

	private String createJSONGrammarWordPair(String jsonGrammar, String word) {
		StringBuilder sb = new StringBuilder();
		sb.append("\"grammar\":");
		sb.append(jsonGrammar);
		sb.append(",\"word\":");
		sb.append("\"" + word + "\"");
		return sb.toString();
	}

	private void writeJSONToFile(List<String> jsons) {
		try (BufferedWriter bw = new BufferedWriter(new FileWriter("src/writer/json.tmp"))) {
			int i = 1;
			for (String json : jsons) {
				bw.write(json);
				bw.newLine();
				bw.newLine();
				if (i % DISTINCT_WORDS == 0) {
					bw.newLine();
				}
				i++;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void writeResultToFile(List<Grammar> grammars, List<String> words, int type) {
		try (BufferedWriter bw = new BufferedWriter(new FileWriter("src/writer/results.tmp"))) {
			int i = 0;
			for (Grammar g : grammars) {
				for (int j = 0; j < DISTINCT_WORDS; j++) {
					if (type == 2) {
						bw.write(t2p.isPartOfLanguage(g.getRules(), words.get(i++), g.getStart()) ? "true" : "false");
					} else {
						bw.write(t3p.isPartOfLanguage(g.getRules(), words.get(i++), g.getStart()) ? "true" : "false");
					}
					bw.newLine();
				}
				bw.newLine();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void runType2Grammar(String[] vars, int alphaSize, int maxConclusios, int maxConclusioSize, String start,
			int mode, int n) {
		List<String> jsonGrammarWordPairs = new LinkedList<>();
		List<Grammar> grammars = new LinkedList<>();
		List<String> words = new LinkedList<>();
		for (int i = 0; i < n; i++) {
			Type2Grammar g = new Type2Grammar(vars, alphaSize, start, maxConclusioSize, maxConclusios, mode);
			grammars.add(g);
			Set<String> distinctWords = new HashSet<>();
			int j = 0;
			while (j < DISTINCT_WORDS) {
				String word = wg.wordGenerator(MAX_LENGTH, alphaSize);
				if (!distinctWords.contains(word)) {
					distinctWords.add(word);
					words.add(word);
					jsonGrammarWordPairs.add(createJSONGrammarWordPair(g.toJSON(), word));

					j++;
				}
			}
		}
		writeResultToFile(grammars, words, 2);
		writeJSONToFile(jsonGrammarWordPairs);
	}

	public static void main(String[] args) {
		Writer w = new Writer();
		String[] vars = { "S", "A", "B" };
		int alphaSize = 2;
		int maxConclusios = 2;
		int maxConclusioSize = 3;
		String start = "S";
		int mode = 3;
		int n = 3;
		w.runType2Grammar(vars, alphaSize, maxConclusios, maxConclusioSize, start, mode, n);
	}

}
